/********************************************************
 * Data Visualization - 2D Country Chart                *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Kimberly Schlesinger                         *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Visualize interactions of library patrons   *
 * with other countries in each Dewey category.         *
 ********************************************************/

// Layout variables 
float constantHorizontalMargin = 160;
float constantVerticalMargin = 100;

// 2D matrix table related variables
Table table;
Table countries;
int numRows, numColumns;
int dispRows = 0;
int noYears = 11;
float[][] dataMatrix;
float maxValue, minValue;
int defaultDeweyClass = 900;
int noInClass = 10;
int yearFrac = 1;
String color_file = "bwReBlend129h.jpg";

// width and height for each cell
float cellWidth, cellHeight;

// text labels
int startYear = 2006;

//interaction
boolean normalisation = false;
boolean colorApply = false;
boolean reorder = false;

void setup() {
  size(1280, 720); // setup the size of the window
  // In Processing 3, the size() function must be the first line of code inside setup()
  // and needs to use numbers instead of variables. 

  // Make the screen resizable.                     
  surface.setResizable(true);

  //Put data into Processing table from csv. Default Dewey category = 900s
  countries = loadTable("q3_countries.csv");
  table = loadTable("q3_country_dewey_yr.csv", "header");
  numRows = table.getRowCount(); 
  numColumns = table.getColumnCount();
  for (TableRow row : table.matchRows("9..","deweyBin")) {
    //int dewBin = row.getInt("deweyBin");
    //println(dewBin);
    //switch(row.getInt("deweyBin") - defaultDeweyClass) {
    //  case 00:
    //  case 10:
    //  case 20:
    //  case 30:
    //  case 40:
    //  case 50:
    //  case 60:
    //  case 70:
    //  case 80:
    //  case 90:
    //}
    dispRows += 1;
  };
  dataMatrix = new float[dispRows][numColumns-2];
  int nc = numColumns-2;
  println("Rows: " + dispRows + " Columns: " + nc);
  
  // Put data into dataMatrix...
  int j = 0;
  for(TableRow row : table.matchRows("9..","deweyBin"))
  {
    for(int i=0; i<numColumns-2; i++)
    {
      dataMatrix[j][i] = row.getFloat(i+2);
    }
    j++;
  }
  
  maxValue = max2D(dataMatrix);
  minValue = min2D(dataMatrix);
  
  println("Max Value is " + maxValue);
  println("Min Value is " + minValue);
}

void draw() {
  
  //refresh the canvas every frame
  background(250);
  
  //modifying the cell size according to window size
  cellWidth = (width - constantHorizontalMargin*2)/(numColumns-2);
  cellHeight = (height - constantVerticalMargin*2)/dispRows;
      
  //Draw 2D Matrix
  //PImage color_image = loadImage(color_file);
  for(int i=0; i<numColumns-2; i++){
    for(int j=0; j<dispRows; j++){
      stroke(255);
      strokeWeight(1);
      //color rectColor = color_image.get((int)map(dataMatrix[j][i], minValue, maxValue, 2, color_image.width-2), color_image.height/2);
      //fill(rectColor);
      float greyScale = map(dataMatrix[j][i], minValue, maxValue, 255, 0);
      fill(greyScale);
      rect(constantHorizontalMargin + cellWidth*i, constantVerticalMargin 
           + cellHeight*j, cellWidth*yearFrac, cellHeight);
    }
  }

  //Draw vertical labels
  for(int i=0; i<numColumns-2; i++) {
    textAlign(CENTER, CENTER);
    fill(0);
    textSize(10);
    String cname = countries.getString(0,i+2);
    text(cname, constantHorizontalMargin + cellWidth/2 + cellWidth*i,
         constantVerticalMargin - cellHeight*2);
  }
  
  // Draw horizontal labels
  for(int i=0; i<noInClass; i++) {
    textAlign(RIGHT,CENTER);
    fill(0);
    textSize(10);
    text(defaultDeweyClass+i*10, 
         constantHorizontalMargin - cellWidth/4,
         constantVerticalMargin + cellHeight*(noYears+1)/2 + cellHeight*(noYears+1)*i);
  }
  //for(int i=0; i<noYears; i++) {
  //  textAlign(RIGHT,CENTER);
  //  fill(0);
  //  textSize(10);
  //  text(startYear+i, 
  //       constantHorizontalMargin - cellWidth/4,
  //       constantVerticalMargin + cellHeight*noYears/2 + cellHeight*noYears*i);
  //}
  
       
  //Draw Title
  textAlign(CENTER, CENTER);
  textSize(16);
  text("country interactions " + defaultDeweyClass, width/2, constantVerticalMargin/2);
  
  //println("Normalisation : " + normalisation + " ColorApply : " + colorApply + " Reorder : " + reorder);
  
  //if (normalisation == true) normalise();
  //else if (normalisation == false) reset();
  
  //if (colorApply == true) colorScheme();
  //else if (colorApply == false) ;
  
  //if (reorder == true) resort();
  //else if (reorder == false) reset(); 
    
}